@@ -41,17 +41,19 @@
 #include "gdm-settings-keys.h"
 
 #include "gdm-slave-proxy.h"
-
-static guint32 display_serial = 1;
+#include "gdm-slave-glue.h"
+#include "gdm-dbus-util.h"
 
 #define GDM_DISPLAY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_DISPLAY, GdmDisplayPrivate))
 
+#define GDM_SLAVE_PATH "/org/gnome/DisplayManager/Slave"
 #define DEFAULT_SLAVE_COMMAND LIBEXECDIR "/gdm-simple-slave"
 
 struct GdmDisplayPrivate
 {
         char                 *id;
         char                 *seat_id;
+        char                 *session_id;
 
         char                 *remote_hostname;
         int                   x11_display_number;
@@ -69,8 +71,14 @@ struct GdmDisplayPrivate
         guint                 finish_idle_id;
 
         GdmSlaveProxy        *slave_proxy;
-        DBusGConnection      *connection;
+        char                 *slave_bus_name;
+        GdmDBusSlave         *slave_bus_proxy;
+        int                   slave_name_id;
+        GDBusConnection      *connection;
         GdmDisplayAccessFile *user_access_file;
+
+        GdmDBusDisplay       *display_skeleton;
+        GDBusObjectSkeleton  *object_skeleton;
 };
 
 enum {
@@ -78,6 +86,7 @@ enum {
         PROP_ID,
         PROP_STATUS,
         PROP_SEAT_ID,
+        PROP_SESSION_ID,
         PROP_REMOTE_HOSTNAME,
         PROP_X11_DISPLAY_NUMBER,
         PROP_X11_DISPLAY_NAME,
@@ -90,6 +99,7 @@ enum {
 static void     gdm_display_class_init  (GdmDisplayClass *klass);
 static void     gdm_display_init        (GdmDisplay      *display);
 static void     gdm_display_finalize    (GObject         *object);
+static void     queue_finish            (GdmDisplay      *display);
 
 G_DEFINE_ABSTRACT_TYPE (GdmDisplay, gdm_display, G_TYPE_OBJECT)
 
@@ -104,20 +114,6 @@ gdm_display_error_quark (void)
         return ret;
 }
 
-static guint32
-get_next_display_serial (void)
-{
-        guint32 serial;
-
-        serial = display_serial++;
-
-        if ((gint32)display_serial < 0) {
-                display_serial = 1;
-        }
-
-        return serial;
-}
-
 time_t
 gdm_display_get_creation_time (GdmDisplay *display)
 {
@@ -134,6 +130,12 @@ gdm_display_get_status (GdmDisplay *display)
         return display->priv->status;
 }
 
+char *
+gdm_display_get_session_id (GdmDisplay *display)
+{
+        return g_strdup (display->priv->session_id);
+}
+
 static GdmDisplayAccessFile *
 _create_access_file_for_user (GdmDisplay  *display,
                               const char  *username,
@@ -275,11 +277,45 @@ gdm_display_add_user_authorization (GdmDisplay *display,
         return ret;
 }
 
+static void
+on_name_vanished (GDBusConnection *connection,
+                  const char      *name,
+                  gpointer         user_data)
+{
+        queue_finish (GDM_DISPLAY (user_data));
+}
+
 static gboolean
 gdm_display_real_set_slave_bus_name (GdmDisplay *display,
                                      const char *name,
                                      GError    **error)
 {
+        g_free (display->priv->slave_bus_name);
+        display->priv->slave_bus_name = g_strdup (name);
+
+        if (display->priv->slave_name_id > 0) {
+                g_bus_unwatch_name (display->priv->slave_name_id);
+        }
+
+        g_bus_watch_name_on_connection (display->priv->connection,
+                                        name,
+                                        G_BUS_NAME_WATCHER_FLAGS_NONE,
+                                        NULL, /* name appeared */
+                                        on_name_vanished,
+                                        g_object_ref (display),
+                                        NULL);
+
+        g_clear_object (&display->priv->slave_bus_proxy);
+        display->priv->slave_bus_proxy = GDM_DBUS_SLAVE (gdm_dbus_slave_proxy_new_sync (display->priv->connection,
+                                                                                        G_DBUS_PROXY_FLAGS_NONE,
+                                                                                        name,
+                                                                                        GDM_SLAVE_PATH,
+                                                                                        NULL, NULL));
+        g_object_bind_property (G_OBJECT (display->priv->slave_bus_proxy),
+                                "session-id",
+                                G_OBJECT (display),
+                                "session-id",
+                                G_BINDING_DEFAULT);
 
         return TRUE;
 }
@@ -431,7 +467,7 @@ gdm_display_get_x11_cookie (GdmDisplay *display,
         g_return_val_if_fail (GDM_IS_DISPLAY (display), FALSE);
 
         if (x11_cookie != NULL) {
-                *x11_cookie = g_array_new (FALSE, FALSE, sizeof (char));
+                *x11_cookie = g_array_new (TRUE, FALSE, sizeof (char));
                 g_array_append_vals (*x11_cookie,
                                      display->priv->x11_cookie,
                                      display->priv->x11_cookie_size);
@@ -795,6 +831,14 @@ _gdm_display_set_seat_id (GdmDisplay     *display,
         display->priv->seat_id = g_strdup (seat_id);
 }
 
+static void
+_gdm_display_set_session_id (GdmDisplay     *display,
+                             const char     *session_id)
+{
+        g_free (display->priv->session_id);
+        display->priv->session_id = g_strdup (session_id);
+}
+
 static void
 _gdm_display_set_remote_hostname (GdmDisplay     *display,
                                   const char     *hostname)
@@ -861,6 +905,9 @@ gdm_display_set_property (GObject        *object,
         case PROP_SEAT_ID:
                 _gdm_display_set_seat_id (self, g_value_get_string (value));
                 break;
+        case PROP_SESSION_ID:
+                _gdm_display_set_session_id (self, g_value_get_string (value));
+                break;
         case PROP_REMOTE_HOSTNAME:
                 _gdm_display_set_remote_hostname (self, g_value_get_string (value));
                 break;
@@ -905,6 +952,9 @@ gdm_display_get_property (GObject        *object,
         case PROP_SEAT_ID:
                 g_value_set_string (value, self->priv->seat_id);
                 break;
+        case PROP_SESSION_ID:
+                g_value_set_string (value, self->priv->session_id);
+                break;
         case PROP_REMOTE_HOSTNAME:
                 g_value_set_string (value, self->priv->remote_hostname);
                 break;
@@ -933,28 +983,345 @@ gdm_display_get_property (GObject        *object,
         }
 }
 
+static gboolean
+handle_get_id (GdmDBusDisplay        *skeleton,
+               GDBusMethodInvocation *invocation,
+               GdmDisplay            *display)
+{
+        char *id;
+
+        gdm_display_get_id (display, &id, NULL);
+
+        gdm_dbus_display_complete_get_id (skeleton, invocation, id);
+
+        g_free (id);
+        return TRUE;
+}
+
+static gboolean
+handle_get_remote_hostname (GdmDBusDisplay        *skeleton,
+                            GDBusMethodInvocation *invocation,
+                            GdmDisplay            *display)
+{
+        char *hostname;
+
+        gdm_display_get_remote_hostname (display, &hostname, NULL);
+
+        gdm_dbus_display_complete_get_remote_hostname (skeleton,
+                                                       invocation,
+                                                       hostname ? hostname : "");
+
+        g_free (hostname);
+        return TRUE;
+}
+
+static gboolean
+handle_get_seat_id (GdmDBusDisplay        *skeleton,
+                    GDBusMethodInvocation *invocation,
+                    GdmDisplay            *display)
+{
+        char *seat_id;
+
+        seat_id = NULL;
+        gdm_display_get_seat_id (display, &seat_id, NULL);
+
+        if (seat_id == NULL) {
+                seat_id = g_strdup ("");
+        }
+        gdm_dbus_display_complete_get_seat_id (skeleton, invocation, seat_id);
+
+        g_free (seat_id);
+        return TRUE;
+}
+
+static gboolean
+handle_get_timed_login_details (GdmDBusDisplay        *skeleton,
+                                GDBusMethodInvocation *invocation,
+                                GdmDisplay            *display)
+{
+        gboolean enabled;
+        char *username;
+        int delay;
+
+        gdm_display_get_timed_login_details (display, &enabled, &username, &delay, NULL);
+
+        gdm_dbus_display_complete_get_timed_login_details (skeleton,
+                                                           invocation,
+                                                           enabled,
+                                                           username ? username : "",
+                                                           delay);
+
+        g_free (username);
+        return TRUE;
+}
+
+static gboolean
+handle_get_x11_authority_file (GdmDBusDisplay        *skeleton,
+                               GDBusMethodInvocation *invocation,
+                               GdmDisplay            *display)
+{
+        char *file;
+
+        gdm_display_get_x11_authority_file (display, &file, NULL);
+
+        gdm_dbus_display_complete_get_x11_authority_file (skeleton, invocation, file);
+
+        g_free (file);
+        return TRUE;
+}
+
+static gboolean
+handle_get_x11_cookie (GdmDBusDisplay        *skeleton,
+                       GDBusMethodInvocation *invocation,
+                       GdmDisplay            *display)
+{
+        GArray *cookie = NULL;
+
+        gdm_display_get_x11_cookie (display, &cookie, NULL);
+
+        gdm_dbus_display_complete_get_x11_cookie (skeleton, invocation, cookie->data);
+
+        g_array_unref (cookie);
+        return TRUE;
+}
+
+static gboolean
+handle_get_x11_display_name (GdmDBusDisplay        *skeleton,
+                             GDBusMethodInvocation *invocation,
+                             GdmDisplay            *display)
+{
+        char *name;
+
+        gdm_display_get_x11_display_name (display, &name, NULL);
+
+        gdm_dbus_display_complete_get_x11_display_name (skeleton, invocation, name);
+
+        g_free (name);
+        return TRUE;
+}
+
+static gboolean
+handle_get_x11_display_number (GdmDBusDisplay        *skeleton,
+                               GDBusMethodInvocation *invocation,
+                               GdmDisplay            *display)
+{
+        int name;
+
+        gdm_display_get_x11_display_number (display, &name, NULL);
+
+        gdm_dbus_display_complete_get_x11_display_number (skeleton, invocation, name);
+
+        return TRUE;
+}
+
+static gboolean
+handle_is_local (GdmDBusDisplay        *skeleton,
+                 GDBusMethodInvocation *invocation,
+                 GdmDisplay            *display)
+{
+        gboolean is_local;
+
+        gdm_display_is_local (display, &is_local, NULL);
+
+        gdm_dbus_display_complete_is_local (skeleton, invocation, is_local);
+
+        return TRUE;
+}
+
+static gboolean
+handle_get_slave_bus_name (GdmDBusDisplay        *skeleton,
+                           GDBusMethodInvocation *invocation,
+                           GdmDisplay            *display)
+{
+        if (display->priv->slave_bus_name != NULL) {
+                gdm_dbus_display_complete_get_slave_bus_name (skeleton, invocation,
+                                                              display->priv->slave_bus_name);
+        } else {
+                g_dbus_method_invocation_return_dbus_error (invocation,
+                                                            "org.gnome.DisplayManager.NotReady",
+                                                            "Slave is not yet registered");
+        }
+
+        return TRUE;
+}
+
+static gboolean
+handle_set_slave_bus_name (GdmDBusDisplay        *skeleton,
+                           GDBusMethodInvocation *invocation,
+                           const char            *bus_name,
+                           GdmDisplay            *display)
+{
+        GError *error;
+
+        if (gdm_display_set_slave_bus_name (display, bus_name, &error)) {
+                gdm_dbus_display_complete_set_slave_bus_name (skeleton, invocation);
+        } else {
+                g_dbus_method_invocation_return_gerror (invocation, error);
+                g_error_free (error);
+        }
+
+        return TRUE;
+}
+
+static gboolean
+handle_add_user_authorization (GdmDBusDisplay        *skeleton,
+                               GDBusMethodInvocation *invocation,
+                               const char            *username,
+                               GdmDisplay            *display)
+{
+        char *filename;
+        GError *error;
+
+        if (gdm_display_add_user_authorization (display, username, &filename, &error)) {
+                gdm_dbus_display_complete_add_user_authorization (skeleton,
+                                                                  invocation,
+                                                                  filename);
+                g_free (filename);
+        } else {
+                g_dbus_method_invocation_return_gerror (invocation, error);
+                g_error_free (error);
+        }
+
+        return TRUE;
+}
+
+static gboolean
+handle_remove_user_authorization (GdmDBusDisplay        *skeleton,
+                                  GDBusMethodInvocation *invocation,
+                                  const char            *username,
+                                  GdmDisplay            *display)
+{
+        GError *error;
+
+        if (gdm_display_remove_user_authorization (display, username, &error)) {
+                gdm_dbus_display_complete_remove_user_authorization (skeleton, invocation);
+        } else {
+                g_dbus_method_invocation_return_gerror (invocation, error);
+                g_error_free (error);
+        }
+
+        return TRUE;
+}
+
 static gboolean
 register_display (GdmDisplay *display)
 {
         GError *error = NULL;
 
         error = NULL;
-        display->priv->connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
+        display->priv->connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
         if (display->priv->connection == NULL) {
-                if (error != NULL) {
-                        g_critical ("error getting system bus: %s", error->message);
-                        g_error_free (error);
-                }
+                g_critical ("error getting system bus: %s", error->message);
+                g_error_free (error);
                 exit (1);
         }
 
-        dbus_g_connection_register_g_object (display->priv->connection, display->priv->id, G_OBJECT (display));
+        display->priv->object_skeleton = g_dbus_object_skeleton_new (display->priv->id);
+        display->priv->display_skeleton = GDM_DBUS_DISPLAY (gdm_dbus_display_skeleton_new ());
+
+        g_signal_connect (display->priv->display_skeleton, "handle-get-id",
+                          G_CALLBACK (handle_get_id), display);
+        g_signal_connect (display->priv->display_skeleton, "handle-get-remote-hostname",
+                          G_CALLBACK (handle_get_remote_hostname), display);
+        g_signal_connect (display->priv->display_skeleton, "handle-get-seat-id",
+                          G_CALLBACK (handle_get_seat_id), display);
+        g_signal_connect (display->priv->display_skeleton, "handle-get-timed-login-details",
+                          G_CALLBACK (handle_get_timed_login_details), display);
+        g_signal_connect (display->priv->display_skeleton, "handle-get-x11-authority-file",
+                          G_CALLBACK (handle_get_x11_authority_file), display);
+        g_signal_connect (display->priv->display_skeleton, "handle-get-x11-cookie",
+                          G_CALLBACK (handle_get_x11_cookie), display);
+        g_signal_connect (display->priv->display_skeleton, "handle-get-x11-display-name",
+                          G_CALLBACK (handle_get_x11_display_name), display);
+        g_signal_connect (display->priv->display_skeleton, "handle-get-x11-display-number",
+                          G_CALLBACK (handle_get_x11_display_number), display);
+        g_signal_connect (display->priv->display_skeleton, "handle-is-local",
+                          G_CALLBACK (handle_is_local), display);
+        g_signal_connect (display->priv->display_skeleton, "handle-get-slave-bus-name",
+                          G_CALLBACK (handle_get_slave_bus_name), display);
+        g_signal_connect (display->priv->display_skeleton, "handle-set-slave-bus-name",
+                          G_CALLBACK (handle_set_slave_bus_name), display);
+        g_signal_connect (display->priv->display_skeleton, "handle-add-user-authorization",
+                          G_CALLBACK (handle_add_user_authorization), display);
+        g_signal_connect (display->priv->display_skeleton, "handle-remove-user-authorization",
+                          G_CALLBACK (handle_remove_user_authorization), display);
+
+        g_dbus_object_skeleton_add_interface (display->priv->object_skeleton,
+                                              G_DBUS_INTERFACE_SKELETON (display->priv->display_skeleton));
 
         return TRUE;
 }
 
+char *
+gdm_display_open_session_sync (GdmDisplay    *display,
+                               GPid           pid_of_caller,
+                               uid_t          uid_of_caller,
+                               GCancellable  *cancellable,
+                               GError       **error)
+{
+        char *address;
+        int ret;
+
+        if (display->priv->slave_bus_proxy == NULL) {
+                g_set_error (error,
+                             G_DBUS_ERROR,
+                             G_DBUS_ERROR_ACCESS_DENIED,
+                             _("No session available yet"));
+                return NULL;
+        }
+
+        address = NULL;
+        ret = gdm_dbus_slave_call_open_session_sync (display->priv->slave_bus_proxy,
+                                                     (int) pid_of_caller,
+                                                     (int) uid_of_caller,
+                                                     &address,
+                                                     cancellable,
+                                                     error);
+
+        if (!ret) {
+                return NULL;
+        }
+
+        return address;
+}
+
+char *
+gdm_display_open_reauthentication_channel_sync (GdmDisplay    *display,
+                                                const char    *username,
+                                                GPid           pid_of_caller,
+                                                uid_t          uid_of_caller,
+                                                GCancellable  *cancellable,
+                                                GError       **error)
+{
+        char *address;
+        int ret;
+
+        if (display->priv->slave_bus_proxy == NULL) {
+                g_set_error (error,
+                             G_DBUS_ERROR,
+                             G_DBUS_ERROR_ACCESS_DENIED,
+                             _("No session available yet"));
+                return NULL;
+        }
+
+        address = NULL;
+        ret = gdm_dbus_slave_call_open_reauthentication_channel_sync (display->priv->slave_bus_proxy,
+                                                                      username,
+                                                                      pid_of_caller,
+                                                                      uid_of_caller,
+                                                                      &address,
+                                                                      cancellable,
+                                                                      error);
+
+        if (!ret) {
+                return NULL;
+        }
+
+        return address;
+}
+
 /*
-  dbus-send --system --print-reply --dest=org.gnome.DisplayManager /org/gnome/DisplayManager/Display1 org.freedesktop.DBus.Introspectable.Introspect
+  dbus-send --system --print-reply --dest=org.gnome.DisplayManager /org/gnome/DisplayManager/Displays/1 org.freedesktop.DBus.Introspectable.Introspect
 */
 
 static GObject *
@@ -963,14 +1330,21 @@ gdm_display_constructor (GType                  type,
                          GObjectConstructParam *construct_properties)
 {
         GdmDisplay      *display;
+        char            *canonical_display_name;
         gboolean         res;
 
         display = GDM_DISPLAY (G_OBJECT_CLASS (gdm_display_parent_class)->constructor (type,
                                                                                        n_construct_properties,
                                                                                        construct_properties));
 
+        canonical_display_name = g_strdelimit (g_strdup (display->priv->x11_display_name),
+                                               ":" G_STR_DELIMITERS, '_');
+
         g_free (display->priv->id);
-        display->priv->id = g_strdup_printf ("/org/gnome/DisplayManager/Display%u", get_next_display_serial ());
+        display->priv->id = g_strdup_printf ("/org/gnome/DisplayManager/Displays/%s",
+                                             canonical_display_name);
+
+        g_free (canonical_display_name);
 
         res = register_display (display);
         if (! res) {
@@ -995,6 +1369,8 @@ gdm_display_dispose (GObject *object)
         }
 
         if (display->priv->slave_proxy != NULL) {
+                gdm_slave_proxy_stop (display->priv->slave_proxy);
+
                 g_object_unref (display->priv->slave_proxy);
                 display->priv->slave_proxy = NULL;
         }
@@ -1011,6 +1387,11 @@ gdm_display_dispose (GObject *object)
                 display->priv->access_file = NULL;
         }
 
+        if (display->priv->slave_name_id > 0) {
+                g_bus_unwatch_name (display->priv->slave_name_id);
+                display->priv->slave_name_id = 0;
+        }
+
         G_OBJECT_CLASS (gdm_display_parent_class)->dispose (object);
 }
 
@@ -1072,6 +1453,13 @@ gdm_display_class_init (GdmDisplayClass *klass)
                                                               "seat id",
                                                               NULL,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+        g_object_class_install_property (object_class,
+                                         PROP_SESSION_ID,
+                                         g_param_spec_string ("session-id",
+                                                              "session id",
+                                                              "session id",
+                                                              NULL,
+                                                              G_PARAM_READWRITE));
         g_object_class_install_property (object_class,
                                          PROP_X11_COOKIE,
                                          g_param_spec_string ("x11-cookie",
@@ -1113,8 +1501,6 @@ gdm_display_class_init (GdmDisplayClass *klass)
                                                            G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
         g_type_class_add_private (klass, sizeof (GdmDisplayPrivate));
-
-        dbus_g_object_type_install_info (GDM_TYPE_DISPLAY, &dbus_glib_gdm_display_object_info);
 }
 
 static void
@@ -1146,6 +1532,11 @@ gdm_display_finalize (GObject *object)
         g_free (display->priv->x11_display_name);
         g_free (display->priv->x11_cookie);
         g_free (display->priv->slave_command);
+        g_free (display->priv->slave_bus_name);
+
+        g_clear_object (&display->priv->display_skeleton);
+        g_clear_object (&display->priv->object_skeleton);
+        g_clear_object (&display->priv->connection);
 
         if (display->priv->access_file != NULL) {
                 g_object_unref (display->priv->access_file);
@@ -1161,3 +1552,15 @@ gdm_display_finalize (GObject *object)
 
         G_OBJECT_CLASS (gdm_display_parent_class)->finalize (object);
 }
+
+GDBusConnection *
+gdm_display_get_bus_connection (GdmDisplay *display)
+{
+        return display->priv->connection;
+}
+
+GDBusObjectSkeleton *
+gdm_display_get_object_skeleton (GdmDisplay *display)
+{
+        return display->priv->object_skeleton;
+}
