@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
 
@@ -40,7 +40,6 @@ typedef enum {
         GDM_DISPLAY_UNMANAGED = 0,
         GDM_DISPLAY_PREPARED,
         GDM_DISPLAY_MANAGED,
-        GDM_DISPLAY_WAITING_TO_FINISH,
         GDM_DISPLAY_FINISHED,
         GDM_DISPLAY_FAILED,
 } GdmDisplayStatus;
@@ -56,8 +55,25 @@ typedef struct
         GObjectClass   parent_class;
 
         /* methods */
-        gboolean (*prepare) (GdmDisplay *display);
-        void     (*manage)  (GdmDisplay *self);
+        gboolean (*create_authority)          (GdmDisplay *display);
+        gboolean (*add_user_authorization)    (GdmDisplay *display,
+                                               const char *username,
+                                               char      **filename,
+                                               GError    **error);
+        gboolean (*remove_user_authorization) (GdmDisplay *display,
+                                               const char *username,
+                                               GError    **error);
+        gboolean (*set_slave_bus_name)        (GdmDisplay *display,
+                                               const char *name,
+                                               GError    **error);
+        gboolean (*prepare)                   (GdmDisplay *display);
+        gboolean (*manage)                    (GdmDisplay *display);
+        gboolean (*finish)                    (GdmDisplay *display);
+        gboolean (*unmanage)                  (GdmDisplay *display);
+        void     (*get_timed_login_details)   (GdmDisplay *display,
+                                               gboolean   *enabled,
+                                               char      **username,
+                                               int        *delay);
 } GdmDisplayClass;
 
 typedef enum
@@ -86,13 +102,14 @@ char *              gdm_display_open_reauthentication_channel_sync        (GdmDi
                                                                            uid_t          uid_of_caller,
                                                                            GCancellable  *cancellable,
                                                                            GError       **error);
-const char *        gdm_display_get_session_id                 (GdmDisplay *display);
+char *              gdm_display_get_session_id                 (GdmDisplay *display);
 gboolean            gdm_display_create_authority               (GdmDisplay *display);
 gboolean            gdm_display_prepare                        (GdmDisplay *display);
 gboolean            gdm_display_manage                         (GdmDisplay *display);
 gboolean            gdm_display_finish                         (GdmDisplay *display);
 gboolean            gdm_display_unmanage                       (GdmDisplay *display);
 
+GDBusConnection    *gdm_display_get_bus_connection             (GdmDisplay *display);
 GDBusObjectSkeleton *gdm_display_get_object_skeleton           (GdmDisplay *display);
 
 /* exported to bus */
@@ -114,14 +131,16 @@ gboolean            gdm_display_get_seat_id                    (GdmDisplay *disp
 gboolean            gdm_display_is_local                       (GdmDisplay *display,
                                                                 gboolean   *local,
                                                                 GError    **error);
-gboolean            gdm_display_is_initial                     (GdmDisplay  *display,
-                                                                gboolean    *initial,
-                                                                GError     **error);
+gboolean            gdm_display_get_timed_login_details        (GdmDisplay *display,
+                                                                gboolean   *enabled,
+                                                                char      **username,
+                                                                int        *delay,
+                                                                GError    **error);
 
-gboolean            gdm_display_get_x11_cookie                 (GdmDisplay  *display,
-                                                                const char **x11_cookie,
-                                                                gsize       *x11_cookie_size,
-                                                                GError     **error);
+/* exported but protected */
+gboolean            gdm_display_get_x11_cookie                 (GdmDisplay *display,
+                                                                GArray     **x11_cookie,
+                                                                GError    **error);
 gboolean            gdm_display_get_x11_authority_file         (GdmDisplay *display,
                                                                 char      **filename,
                                                                 GError    **error);
@@ -132,10 +151,10 @@ gboolean            gdm_display_add_user_authorization         (GdmDisplay *disp
 gboolean            gdm_display_remove_user_authorization      (GdmDisplay *display,
                                                                 const char *username,
                                                                 GError    **error);
-void                gdm_display_start_greeter_session          (GdmDisplay  *display);
-void                gdm_display_stop_greeter_session           (GdmDisplay  *display);
+gboolean            gdm_display_set_slave_bus_name             (GdmDisplay *display,
+                                                                const char *name,
+                                                                GError    **error);
 
-gboolean            gdm_display_connect                        (GdmDisplay *self);
 
 G_END_DECLS
 
