@@ -26,14 +26,13 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib-object.h>
-
-#include <dbus/dbus.h>
-#include <dbus/dbus-glib-lowlevel.h>
+#include <gio/gio.h>
 
 #ifdef WITH_SYSTEMD
 #include <systemd/sd-daemon.h>
 #endif
 
+#include "gdm-manager.h"
 #include "gdm-display-factory.h"
 #include "gdm-local-display-factory.h"
 #include "gdm-local-display-factory-glue.h"
@@ -41,7 +40,6 @@
 #include "gdm-display-store.h"
 #include "gdm-static-display.h"
 #include "gdm-transient-display.h"
-#include "gdm-product-display.h"
 
 #define GDM_LOCAL_DISPLAY_FACTORY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_LOCAL_DISPLAY_FACTORY, GdmLocalDisplayFactoryPrivate))
 
@@ -56,12 +54,17 @@
 
 struct GdmLocalDisplayFactoryPrivate
 {
-        DBusGConnection *connection;
-        DBusGProxy      *proxy;
+        GdmDBusLocalDisplayFactory *skeleton;
+        GDBusConnection *connection;
         GHashTable      *displays;
 
         /* FIXME: this needs to be per seat? */
         guint            num_failures;
+
+#ifdef WITH_SYSTEMD
+        guint            seat_new_id;
+        guint            seat_removed_id;
+#endif
 };
 
 enum {
@@ -75,6 +78,10 @@ static void     gdm_local_display_factory_finalize      (GObject
 static GdmDisplay *create_display                       (GdmLocalDisplayFactory      *factory,
                                                          const char                  *seat_id);
 
+static void     on_display_status_changed               (GdmDisplay                  *display,
+                                                         GParamSpec                  *arg1,
+                                                         GdmLocalDisplayFactory      *factory);
+
 static gpointer local_display_factory_object = NULL;
 
 G_DEFINE_TYPE (GdmLocalDisplayFactory, gdm_local_display_factory, GDM_TYPE_DISPLAY_FACTORY)
@@ -178,6 +185,9 @@ store_display (GdmLocalDisplayFactory *factory,
 {
         GdmDisplayStore *store;
 
+        g_signal_connect (display, "notify::status",
+                          G_CALLBACK (on_display_status_changed), factory);
+
         g_object_weak_ref (G_OBJECT (display), (GWeakNotify)on_display_disposed, factory);
 
         store = gdm_display_factory_get_display_store (GDM_DISPLAY_FACTORY (factory));
@@ -252,58 +262,10 @@ gdm_local_display_factory_create_transient_display (GdmLocalDisplayFactory *fact
         return ret;
 }
 
-gboolean
-gdm_local_display_factory_create_product_display (GdmLocalDisplayFactory *factory,
-                                                  const char             *parent_display_id,
-                                                  const char             *relay_address,
-                                                  char                  **id,
-                                                  GError                **error)
-{
-        gboolean    ret;
-        GdmDisplay *display;
-        guint32     num;
-        const char *seat_id;
-
-        g_return_val_if_fail (GDM_IS_LOCAL_DISPLAY_FACTORY (factory), FALSE);
-
-        ret = FALSE;
-
-        g_debug ("GdmLocalDisplayFactory: Creating product display parent %s address:%s",
-                 parent_display_id, relay_address);
-
-        num = take_next_display_number (factory);
-
-        g_debug ("GdmLocalDisplayFactory: got display num %u", num);
-
-        display = gdm_product_display_new (num, relay_address);
-
-        seat_id = get_seat_of_transient_display (factory);
-        g_object_set (display, "seat-id", seat_id, NULL);
-
-        store_display (factory, num, display);
-
-        if (! gdm_display_manage (display)) {
-                display = NULL;
-                goto out;
-        }
-
-        if (! gdm_display_get_id (display, id, NULL)) {
-                display = NULL;
-                goto out;
-        }
-
-        ret = TRUE;
- out:
-        /* ref either held by store or not at all */
-        g_object_unref (display);
-
-        return ret;
-}
-
 static void
-on_static_display_status_changed (GdmDisplay             *display,
-                                  GParamSpec             *arg1,
-                                  GdmLocalDisplayFactory *factory)
+on_display_status_changed (GdmDisplay             *display,
+                           GParamSpec             *arg1,
+                           GdmLocalDisplayFactory *factory)
 {
         int              status;
         GdmDisplayStore *store;
@@ -320,29 +282,40 @@ on_static_display_status_changed (GdmDisplay             *display,
 
         status = gdm_display_get_status (display);
 
-        g_debug ("GdmLocalDisplayFactory: static display status changed: %d", status);
+        g_debug ("GdmLocalDisplayFactory: display status changed: %d", status);
         switch (status) {
         case GDM_DISPLAY_FINISHED:
                 /* remove the display number from factory->priv->displays
                    so that it may be reused */
                 g_hash_table_remove (factory->priv->displays, GUINT_TO_POINTER (num));
                 gdm_display_store_remove (store, display);
-                /* reset num failures */
-                factory->priv->num_failures = 0;
-                create_display (factory, seat_id);
+
+                /* Create a new equivalent display if it was static */
+                if (GDM_IS_STATIC_DISPLAY (display)) {
+                        /* reset num failures */
+                        factory->priv->num_failures = 0;
+
+                        create_display (factory, seat_id);
+                }
                 break;
         case GDM_DISPLAY_FAILED:
                 /* leave the display number in factory->priv->displays
                    so that it doesn't get reused */
                 gdm_display_store_remove (store, display);
-                factory->priv->num_failures++;
-                if (factory->priv->num_failures > MAX_DISPLAY_FAILURES) {
-                        /* oh shit */
-                        g_warning ("GdmLocalDisplayFactory: maximum number of X display failures reached: check X server log for errors");
-                        /* FIXME: should monitor hardware changes to
-                           try again when seats change */
-                } else {
-                        create_display (factory, seat_id);
+
+                /* Create a new equivalent display if it was static */
+                if (GDM_IS_STATIC_DISPLAY (display)) {
+
+                        factory->priv->num_failures++;
+
+                        if (factory->priv->num_failures > MAX_DISPLAY_FAILURES) {
+                                /* oh shit */
+                                g_warning ("GdmLocalDisplayFactory: maximum number of X display failures reached: check X server log for errors");
+                                /* FIXME: should monitor hardware changes to
+                                   try again when seats change */
+                        } else {
+                                create_display (factory, seat_id);
+                        }
                 }
                 break;
         case GDM_DISPLAY_UNMANAGED:
@@ -396,19 +369,10 @@ create_display (GdmLocalDisplayFactory *factory,
 
         num = take_next_display_number (factory);
 
-#if 0
-        display = gdm_static_factory_display_new (num);
-#else
         display = gdm_static_display_new (num);
-#endif
 
         g_object_set (display, "seat-id", seat_id, NULL);
 
-        g_signal_connect (display,
-                          "notify::status",
-                          G_CALLBACK (on_static_display_status_changed),
-                          factory);
-
         store_display (factory, num, display);
 
         /* let store own the ref */
@@ -437,143 +401,108 @@ delete_display (GdmLocalDisplayFactory *factory,
 
 static gboolean gdm_local_display_factory_sync_seats (GdmLocalDisplayFactory *factory)
 {
-        DBusError error;
-        DBusMessage *message, *reply;
-        DBusMessageIter iter, sub, sub2;
+        GError *error = NULL;
+        GVariant *result;
+        GVariant *array;
+        GVariantIter iter;
+        const char *seat;
 
-        dbus_error_init (&error);
+        result = g_dbus_connection_call_sync (factory->priv->connection,
+                                              "org.freedesktop.login1",
+                                              "/org/freedesktop/login1",
+                                              "org.freedesktop.login1.Manager",
+                                              "ListSeats",
+                                              NULL,
+                                              G_VARIANT_TYPE ("(a(so))"),
+                                              G_DBUS_CALL_FLAGS_NONE,
+                                              -1,
+                                              NULL, &error);
 
-        message = dbus_message_new_method_call (
-                        "org.freedesktop.login1",
-                        "/org/freedesktop/login1",
-                        "org.freedesktop.login1.Manager",
-                        "ListSeats");
-        if (message == NULL) {
-                g_warning ("GdmLocalDisplayFactory: Failed to allocate message");
+        if (!result) {
+                g_warning ("GdmLocalDisplayFactory: Failed to issue method call: %s", error->message);
+                g_clear_error (&error);
                 return FALSE;
         }
 
-        reply = dbus_connection_send_with_reply_and_block (dbus_g_connection_get_connection (factory->priv->connection), message, -1, &error);
-        dbus_message_unref (message);
+        array = g_variant_get_child_value (result, 0);
+        g_variant_iter_init (&iter, array);
 
-        if (reply == NULL) {
-                g_warning ("GdmLocalDisplayFactory: Failed to issue method call: %s", error.message);
-                dbus_error_free (&error);
-                return FALSE;
-        }
-
-        if (!dbus_message_iter_init (reply, &iter) ||
-            dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_ARRAY ||
-            dbus_message_iter_get_element_type (&iter) != DBUS_TYPE_STRUCT)  {
-                g_warning ("GdmLocalDisplayFactory: Failed to parse reply.");
-                dbus_message_unref (reply);
-                return FALSE;
-        }
-
-        dbus_message_iter_recurse (&iter, &sub);
-
-        while (dbus_message_iter_get_arg_type (&sub) != DBUS_TYPE_INVALID) {
-                const char *seat;
-
-                if (dbus_message_iter_get_arg_type (&sub) != DBUS_TYPE_STRUCT) {
-                        g_warning ("GdmLocalDisplayFactory: Failed to parse reply.");
-                        dbus_message_unref (reply);
-                        return FALSE;
-                }
-
-                dbus_message_iter_recurse (&sub, &sub2);
-
-                if (dbus_message_iter_get_arg_type (&sub2) != DBUS_TYPE_STRING) {
-                        g_warning ("GdmLocalDisplayFactory: Failed to parse reply.");
-                        dbus_message_unref (reply);
-                        return FALSE;
-                }
-
-                dbus_message_iter_get_basic (&sub2, &seat);
+        while (g_variant_iter_loop (&iter, "(&so)", &seat, NULL))
                 create_display (factory, seat);
 
-                dbus_message_iter_next (&sub);
-        }
-
-        dbus_message_unref (reply);
+        g_variant_unref (result);
+        g_variant_unref (array);
         return TRUE;
 }
 
-static DBusHandlerResult
-on_seat_signal (DBusConnection *connection,
-                DBusMessage    *message,
-                void           *user_data)
+static void
+on_seat_new (GDBusConnection *connection,
+             const gchar     *sender_name,
+             const gchar     *object_path,
+             const gchar     *interface_name,
+             const gchar     *signal_name,
+             GVariant        *parameters,
+             gpointer         user_data)
 {
-        GdmLocalDisplayFactory *factory = user_data;
-        DBusError error;
+        const char *seat;
 
-        dbus_error_init (&error);
+        g_variant_get (parameters, "(&s)", &seat);
+        create_display (GDM_LOCAL_DISPLAY_FACTORY (user_data), seat);
+}
 
-        if (dbus_message_is_signal (message, "org.freedesktop.login1.Manager", "SeatNew") ||
-            dbus_message_is_signal (message, "org.freedesktop.login1.Manager", "SeatRemoved")) {
-                const char *seat;
+static void
+on_seat_removed (GDBusConnection *connection,
+                 const gchar     *sender_name,
+                 const gchar     *object_path,
+                 const gchar     *interface_name,
+                 const gchar     *signal_name,
+                 GVariant        *parameters,
+                 gpointer         user_data)
+{
+        const char *seat;
 
-                dbus_message_get_args (message,
-                                       &error,
-                                       DBUS_TYPE_STRING, &seat,
-                                       DBUS_TYPE_INVALID);
-
-                if (dbus_error_is_set (&error)) {
-                        g_warning ("GdmLocalDisplayFactory: Failed to decode seat message: %s", error.message);
-                        dbus_error_free (&error);
-                } else {
-
-                        if (strcmp (dbus_message_get_member (message), "SeatNew") == 0) {
-                                create_display (factory, seat);
-                        } else {
-                                delete_display (factory, seat);
-                        }
-                }
-        }
-
-        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+        g_variant_get (parameters, "(&s)", &seat);
+        delete_display (GDM_LOCAL_DISPLAY_FACTORY (user_data), seat);
 }
 
 static void
 gdm_local_display_factory_start_monitor (GdmLocalDisplayFactory *factory)
 {
-        DBusError error;
-
-        dbus_error_init (&error);
-
-        dbus_bus_add_match (dbus_g_connection_get_connection (factory->priv->connection),
-                            "type='signal',"
-                            "sender='org.freedesktop.login1',"
-                            "path='/org/freedesktop/login1',"
-                            "interface='org.freedesktop.login1.Manager',"
-                            "member='SeatNew'",
-                            &error);
-
-        if (dbus_error_is_set (&error)) {
-                g_warning ("GdmLocalDisplayFactory: Failed to add match for SeatNew: %s", error.message);
-                dbus_error_free (&error);
-        }
-
-        dbus_bus_add_match (dbus_g_connection_get_connection (factory->priv->connection),
-                            "type='signal',"
-                            "sender='org.freedesktop.login1',"
-                            "path='/org/freedesktop/login1',"
-                            "interface='org.freedesktop.login1.Manager',"
-                            "member='SeatRemoved'",
-                            &error);
-
-        if (dbus_error_is_set (&error)) {
-                g_warning ("GdmLocalDisplayFactory: Failed to add match for SeatNew: %s", error.message);
-                dbus_error_free (&error);
-        }
-
-        dbus_connection_add_filter (dbus_g_connection_get_connection (factory->priv->connection), on_seat_signal, factory, NULL);
+        factory->priv->seat_new_id = g_dbus_connection_signal_subscribe (factory->priv->connection,
+                                                                         "org.freedesktop.login1",
+                                                                         "org.freedesktop.login1.Manager",
+                                                                         "SeatNew",
+                                                                         "/org/freedesktop/login1",
+                                                                         NULL,
+                                                                         G_DBUS_SIGNAL_FLAGS_NONE,
+                                                                         on_seat_new,
+                                                                         g_object_ref (factory),
+                                                                         g_object_unref);
+        factory->priv->seat_removed_id = g_dbus_connection_signal_subscribe (factory->priv->connection,
+                                                                             "org.freedesktop.login1",
+                                                                             "org.freedesktop.login1.Manager",
+                                                                             "SeatRemoved",
+                                                                             "/org/freedesktop/login1",
+                                                                             NULL,
+                                                                             G_DBUS_SIGNAL_FLAGS_NONE,
+                                                                             on_seat_removed,
+                                                                             g_object_ref (factory),
+                                                                             g_object_unref);
 }
 
 static void
 gdm_local_display_factory_stop_monitor (GdmLocalDisplayFactory *factory)
 {
-        dbus_connection_remove_filter (dbus_g_connection_get_connection (factory->priv->connection), on_seat_signal, factory);
+        if (factory->priv->seat_new_id) {
+                g_dbus_connection_signal_unsubscribe (factory->priv->connection,
+                                                      factory->priv->seat_new_id);
+                factory->priv->seat_new_id = 0;
+        }
+        if (factory->priv->seat_removed_id) {
+                g_dbus_connection_signal_unsubscribe (factory->priv->connection,
+                                                      factory->priv->seat_removed_id);
+                factory->priv->seat_removed_id = 0;
+        }
 }
 
 #endif
@@ -639,22 +568,56 @@ gdm_local_display_factory_get_property (GObject    *object,
         }
 }
 
+static gboolean
+handle_create_transient_display (GdmDBusLocalDisplayFactory *skeleton,
+                                 GDBusMethodInvocation      *invocation,
+                                 GdmLocalDisplayFactory     *factory)
+{
+        GError *error = NULL;
+        gboolean created;
+        char *id = NULL;
+
+        created = gdm_local_display_factory_create_transient_display (factory,
+                                                                      &id,
+                                                                      &error);
+        if (!created) {
+                g_dbus_method_invocation_return_gerror (invocation, error);
+        } else {
+                gdm_dbus_local_display_factory_complete_create_transient_display (skeleton, invocation, id);
+        }
+
+        g_free (id);
+        return TRUE;
+}
+
 static gboolean
 register_factory (GdmLocalDisplayFactory *factory)
 {
         GError *error = NULL;
 
         error = NULL;
-        factory->priv->connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
+        factory->priv->connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
         if (factory->priv->connection == NULL) {
-                if (error != NULL) {
-                        g_critical ("error getting system bus: %s", error->message);
-                        g_error_free (error);
-                }
+                g_critical ("error getting system bus: %s", error->message);
+                g_error_free (error);
                 exit (1);
         }
 
-        dbus_g_connection_register_g_object (factory->priv->connection, GDM_LOCAL_DISPLAY_FACTORY_DBUS_PATH, G_OBJECT (factory));
+        factory->priv->skeleton = GDM_DBUS_LOCAL_DISPLAY_FACTORY (gdm_dbus_local_display_factory_skeleton_new ());
+
+        g_signal_connect (factory->priv->skeleton,
+                          "handle-create-transient-display",
+                          G_CALLBACK (handle_create_transient_display),
+                          factory);
+
+        if (!g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (factory->priv->skeleton),
+                                               factory->priv->connection,
+                                               GDM_LOCAL_DISPLAY_FACTORY_DBUS_PATH,
+                                               &error)) {
+                g_critical ("error exporting LocalDisplayFactory object: %s", error->message);
+                g_error_free (error);
+                exit (1);
+        }
 
         return TRUE;
 }
@@ -694,8 +657,6 @@ gdm_local_display_factory_class_init (GdmLocalDisplayFactoryClass *klass)
         factory_class->stop = gdm_local_display_factory_stop;
 
         g_type_class_add_private (klass, sizeof (GdmLocalDisplayFactoryPrivate));
-
-        dbus_g_object_type_install_info (GDM_TYPE_LOCAL_DISPLAY_FACTORY, &dbus_glib_gdm_local_display_factory_object_info);
 }
 
 static void
@@ -718,6 +679,9 @@ gdm_local_display_factory_finalize (GObject *object)
 
         g_return_if_fail (factory->priv != NULL);
 
+        g_clear_object (&factory->priv->connection);
+        g_clear_object (&factory->priv->skeleton);
+
         g_hash_table_destroy (factory->priv->displays);
 
 #ifdef WITH_SYSTEMD
