@@ -278,17 +278,18 @@ static void
 gdm_server_init_command (GdmServer *server)
 {
         gboolean debug = FALSE;
-        const char *logverbose;
+        const char *debug_options;
 
         if (server->priv->command != NULL) {
                 return;
         }
 
         gdm_settings_direct_get_boolean (GDM_KEY_DEBUG, &debug);
-        if (debug)
-                logverbose = " -logverbose 7";
-        else
-                logverbose = "";
+        if (debug) {
+                debug_options = " -logverbose 7 -core ";
+        } else {
+                debug_options = "";
+        }
 
 #ifdef WITH_SYSTEMD
 
@@ -319,13 +320,13 @@ gdm_server_init_command (GdmServer *server)
                 goto fallback;
         }
 
-        server->priv->command = g_strdup_printf (SYSTEMD_X_SERVER " -br -verbose%s", logverbose);
+        server->priv->command = g_strdup_printf (SYSTEMD_X_SERVER " -br -verbose%s", debug_options);
         return;
 
 fallback:
 #endif
 
-        server->priv->command = g_strdup_printf (X_SERVER " -br -verbose%s", logverbose);
+        server->priv->command = g_strdup_printf (X_SERVER " -br -verbose%s", debug_options);
 }
 
 static gboolean
@@ -799,10 +800,19 @@ gboolean
 gdm_server_start_on_active_vt (GdmServer *server)
 {
         gboolean res;
+        gboolean debug;
         char *vt;
+        const char *debug_options;
+
+        gdm_settings_direct_get_boolean (GDM_KEY_DEBUG, &debug);
+        if (debug) {
+                debug_options = " -logverbose 7 -core ";
+        } else {
+                debug_options = "";
+        }
 
         g_free (server->priv->command);
-        server->priv->command = g_strdup (X_SERVER " -background none -logverbose 7");
+        server->priv->command = g_strdup_printf (X_SERVER " -background none -verbose%s", debug_options);
         vt = get_active_vt_as_string ();
         res = gdm_server_spawn (server, vt);
         g_free (vt);
