@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
 
@@ -59,8 +59,6 @@ void                    gdm_session_worker_job_set_server_address (GdmSessionWor
                                                                    const char          *server_address);
 void                    gdm_session_worker_job_set_for_reauth (GdmSessionWorkerJob *session_worker_job,
                                                                gboolean             for_reauth);
-void                    gdm_session_worker_job_set_environment    (GdmSessionWorkerJob *session_worker_job,
-                                                                   const char * const  *environment);
 gboolean                gdm_session_worker_job_start              (GdmSessionWorkerJob *session_worker_job,
                                                                    const char          *name);
 void                    gdm_session_worker_job_stop               (GdmSessionWorkerJob *session_worker_job);
